{

	// SSA Import V1.0
	// Paul Tuersley, July 2005.

	// change to false if you don't want keyframes to be rounded to the nearest frame.
	var roundKeysToWholeFrames = true;



	// roundToFrames()
	// Receives a time in seconds;
	// Returns time in seconds, rounded so keyframe will land on an exact frame
	//
	function roundToFrames(timeInSecs) {
		if (roundKeysToWholeFrames) { 
			var roundedInSecs = activeItem.frameDuration * (Math.round(timeInSecs / activeItem.frameDuration));
		return(roundedInSecs);
		}
	return(timeInSecs);
	}



	// timeInSecs()
	// Receives a string in the format "00:00:00.00". Hours, minutes, secs, hundreths.
	// Returns a value in seconds.
	//
	function timeInSecs(timeString) {
		var timeArray = timeString.split(":");
		var timeInSecs = (parseInt(timeArray[0]) * 3600) + (parseInt(timeArray[1]) * 60) + parseFloat(timeArray[2]);
	return(timeInSecs);
	}



	// interpretLine()
	// Receives a line that begins with "Dialogue" from an SSA file.
	// Format: Marked, Start, End, Style, Name, MarginL, MarginR, MarginV, Effect, Text
	// Sends Start, End and Text to either the createSubtitle() or createKaraoke() functions.
	//
	function interpretLine(theLine) {

		var textStartIndex = 0;

		// loop through first 9 commas to find where text starts.
		for (var j = 0; j < 9; ++j) {
			textStartIndex = theLine.indexOf(",", textStartIndex) + 1;
		}

		// Split off the text at the end of the line into a separate string
		var theText = theLine.substring(textStartIndex, theLine.length);
		var theInfo = theLine.substring(0, textStartIndex -1);
	
		// Break non-text section into an array, placing relevant data into variables.
		var theInfoArray = theInfo.split(",");
		var startTime = timeInSecs(theInfoArray[startIndex]);
		var endTime = timeInSecs(theInfoArray[endIndex]);
		var theEffect = theInfoArray[effectIndex];

		// Send Start, End and Text to either the createSubtitle() or createKaraoke() functions.
		if (theEffect == "Karaoke") {
			createKaraoke(startTime, endTime,  theText);
		} else {
			createSubtitle(startTime, endTime, theText);
		}

		// Update global that will be used to set minimum length of the comp.
		if (finalEndTime < endTime) {
			finalEndTime = endTime;
		}
	}




	// createKaraoke()
	// Receives startTime, endTime and all the text data.
	// Searches for any text found after {\k35} type overrides (\k = highlight duration, 35 = 0.35 secs)
	// Adds keyframes to sourceText and the range selector's Start and End properties.
	// Returns nothing.
	//
	function createKaraoke(startTime, endTime, theText) {

		var currentTime = startTime;
		var tempArray = new Array();
		var duration = 0;
		var currentWord = "";
		var finalText = "";
		var endIndex;
		var curIndex = 0;

		// search through theText for valid text
		while (curIndex < theText.length) {
			currentWord = "";
			duration = 0;

			// if the character is "{", this is either a duration override (\k) or of no interest
			if (theText.charAt(curIndex) == "{") {
				
				// search for next "}" to find end of override
				endIndex = theText.indexOf("}", curIndex + 1);

				// if there is no "}" later in the text, assume something is wrong and stop reading this line
				if (endIndex == -1) {
					break;

				} else {				
					// if looking at duration info, grab duration
					if (theText.charAt(curIndex+2) == "k") {
						duration = (parseInt(theText.substring(curIndex+3, endIndex))) / 100;
					}
					curIndex = endIndex + 1;
				}
			}

			// search for a { to indicate end of text
			endIndex = theText.indexOf("{", curIndex);

			// if there are no more {'s, assume what's left is probably text, the while loop will finish on this pass.
			if (endIndex == -1) {
				currentWord = theText.substring(curIndex, theText.length)
				curIndex = theText.length;
			} else {
				// assume what comes between these two overrides is text.
				currentWord = theText.substring(curIndex, endIndex)
				curIndex = endIndex;
			}

			// if a duration override was found, add range selector keyframes and add the text to the finalText. 
			if  (duration > 0 && currentWord.length > 0) {
				// add 'Start' keyframes to range selector's Start and End properties.
				rangeStartProp.setValueAtTime(roundToFrames(currentTime), finalText.length);
				rangeEndProp.setValueAtTime(roundToFrames(currentTime),  finalText.length + currentWord.length);
				finalText += currentWord;
			}

			// advance the current time based on any duration overide found (regardless of whether any text was found)
			currentTime += duration;
		}

		// if any karaoke text was found, add sourceText and the final range selector keyframes.
		if (finalText.length > 0) {

			// add the combined text to the sourceText property.
			theTextLayer.sourceText.setValueAtTime(roundToFrames(startTime), finalText);
			theTextLayer.sourceText.setValueAtTime(roundToFrames(endTime), " ");
	
			// add 'End' keyframes to range selector's Start and End properties.
			rangeStartProp.setValueAtTime(roundToFrames(currentTime), 0);
			rangeEndProp.setValueAtTime(roundToFrames(currentTime), 0);

		}
	}





	// createSubtitle()
	// Receives startTime, endTime and all the text data.
	// Ignores all {/} overrides, treats everything else as text.
	// Adds keyframes to sourceText only.
	// Returns nothing.
	//
	function createSubtitle(startTime, endTime, theText) {

		var endIndex;
		var finalText = "";
		var curIndex = 0;

		// search through theText for valid text
		while (curIndex < theText.length) {

			// if the character is "{", skip to end of override
			if (theText.charAt(curIndex) == "{") {

				// find end "}" of override
				curIndex = theText.indexOf("}", curIndex + 1);
		
				// if for some reason no end is found, stop searching this line of text
				// any text already found will still be applied.
				if (curIndex == -1) {
					break;
				}
				curIndex++;

			} else {
				endIndex = theText.indexOf("{", curIndex);
	
				// if there is no "{" later in the text, grab all the remaining text and end loop
				if (endIndex == -1) {
					finalText += theText.substring(curIndex, theText.length);
					break;
				} else {
					//but if there is a { later, grab the text in between and loop
					finalText += theText.substring(curIndex, endIndex);
					curIndex = endIndex;
				}
			}
		}

		// if there is some valid text, add the sourceText keyframes
		if (finalText.length > 0) {
			// add the text to the sourceText property.
			theTextLayer.sourceText.setValueAtTime(roundToFrames(startTime), finalText);
			theTextLayer.sourceText.setValueAtTime(roundToFrames(endTime), " ");
		}

	}



		




	// MAIN SCRIPT

	var activeItem = app.project.activeItem;
	
	// MAKE SURE A SINGLE COMP IS SELECTED.
	if ( !activeItem || !(activeItem instanceof CompItem)) {
		alert("You need to create and select one text layer before running this script");
	} else {

		// AND MAKE SURE A SINGLE TEXT LAYER IS SELECTED
		if (activeItem.selectedLayers.length != 1 || !activeItem.selectedLayers[0].sourceText) {
			alert("You need to create and select one text layer before running this script");
		} else {
	
			// PROMPT FOR .SSA FILE AND OPEN
			var theFile = fileGetDialog("select a .SSA file","");

			if (theFile) {
				theFile.open("r");

				app.beginUndoGroup("Import SSA file");

				// SET UP TEXT LAYER
				var theTextLayer = activeItem.selectedLayers[0];
				// make sure the text layer starts on first frame.
				theTextLayer.startTime = 0;
				theTextLayer.name = "SSA_Import";

				// SET UP ANIMATORS AND SELECTORS
				var theAnimators = theTextLayer.property(1).property(4);

				// set up the default animator
				var defaultAnimator = theAnimators.addProperty("ADBE Text Animator");
				defaultAnimator.name = "Default Animator";

				// set up the highlight animator
				var highlightAnimator = theAnimators.addProperty("ADBE Text Animator");
				highlightAnimator.name = "Highlight Animator";

				// set up highlight selector
				var highlightSelector = highlightAnimator.Selectors.addProperty("ADBE Text Selector");
				var advancedSelector = highlightSelector.property(7);
				var unitsProperty = advancedSelector.property(1);
				unitsProperty.setValue(2);


				// ADD 'HOLD' KEYFRAMES AT COMP START.
				// add first keyframes to sourceText and range selector's Start and End properties.
				var rangeStartProp = highlightSelector.property(4);
				var rangeEndProp = highlightSelector.property(5);
				rangeStartProp.setValueAtTime(0,0);
				rangeEndProp.setValueAtTime(0,0);
				rangeStartProp.setInterpolationTypeAtKey(1,KeyframeInterpolationType.HOLD, KeyframeInterpolationType.HOLD);
				rangeEndProp.setInterpolationTypeAtKey(1,KeyframeInterpolationType.HOLD, KeyframeInterpolationType.HOLD);
				theTextLayer.sourceText.setValueAtTime(0, " ");


				// ADD THE DEFAULT ANIMATOR PROPERTIES
				// define setting for the default animator
				// this currently creates text with a white fill.
				var defaultProperty = defaultAnimator.Properties.addProperty("ADBE Text Fill Color");
				defaultProperty.setValue([1,1,1,0]);

				// define setting for the highlight animator
				// this currently creates text with a red fill
				var highlightProperty = highlightAnimator.Properties.addProperty("ADBE Text Fill Color");


				// define global used to work out minimum comp length.
				var finalEndTime = 0;

				var startIndex, endIndex, effectIndex

				// LOOP THROUGH EACH LINE IN .SSA FILE, CHECKING FOR "FORMAT" OR "DIALOGUE"
				while (theFile.tell() < theFile.length) {
					theLine = theFile.readln();

					// If line begins 'Format', store the indexes for Start, End and Effect
					if (theLine.substring(0,6) == "Format") {
						var formatArray = theLine.split(", ");
							for (var i = 0; i < formatArray.length; ++i) {
								if (formatArray[i] == "Start") { startIndex = i; }
								else if (formatArray[i] == "End") { endIndex = i; }
								else if (formatArray[i] == "Effect") { effectIndex = i; }
							}
					}
	
					// If line begins "Dialogue", send line to the interpretLine() function
					if (theLine.substring(0,8) == "Dialogue") {
						interpretLine(theLine);
					}
				}
				theFile.close();


				// MAKE SURE COMP AND LAYER END AFTER OR ON THE LAST KEYFRAME.
				if (activeItem.duration < finalEndTime) {
					activeItem.duration = finalEndTime;
					theTextLayer.outPoint = finalEndTime;
				}
				app.endUndoGroup();
				clearOutput();	
			}
		}
	}
}




	